;****************************************************************
;		     Hu6280 Helpful Defines			*
;****************************************************************
.define	LO			0
.define	HI			1
.define	BANK			2
.define	MCMAP			3

; Hardware Pages
;----------------
.define BANK_HW			$FF
.define BANK_WRAM		$F8

; Mapping Defines
;-----------------
.define MPR7			%10000000
.define MPR6			%01000000
.define MPR5			%00100000
.define MPR4			%00010000
.define MPR3			%00001000
.define MPR2			%00000100
.define MPR1			%00000010
.define MPR0			%00000001

;****************************************************************
;		     Magic Waveform Generator			*
;****************************************************************
.define PSG_CHANNEL		$0800
; -/W
;	2-0: Current audio channel to interact with
; through registers $0802 to $0809.

.define PSG_GLOPAN		$0801
; -/W
;	7-4: Volume to the left speaker.
;	3-0: Volume to the right speaker.
; This is the global volume, as in the amplitude after all
; channels have been summed. This works like the Lynx's panning
; registers where %1111 is loudest and %0000 is silence.

.define PSG_FREQ_LO		$0802
.define PSG_FREQ_HI		$0803
; -/W
;	11-0: Divider for controlling the waveform playback
; frequency. When this expires, the waveform pointer is
; advanced by one. Frequency = 3.58MHz / (Value)

.define PSG_CONTROL		$0804
; -/W
.define PSG_CONTROL_WRITE	%00000000
.define PSG_CONTROL_RESET	%01000000
.define PSG_CONTROL_WAVE	%10000000
.define PSG_CONTROL_SAMPLE	%11000000
;	7: Channel Output Enable
;	6: DDA Output Mode
.define PSG_CONTROL_VOLMASK	%00011111
;	4-0: Overall channel volume, lower values are quieter.

.define PSG_PAN			$0805
; -/W
;	7-4: Volume to the left speaker.
;	3-0: Volume to the right speaker.
; Works the same way global panning does, but only affects
; the target channel.

.define PSG_WAVEDAT		$0806
; -/W
;	4-0: Sample data to either add to the 32-Byte waveform
; memory (in PSG_CONTROL_WRITE mode), or output directly to
; the stereo mixing hardware (in PSG_CONTROL_SAMPLE mode).

.define PSG_LFSR		$0807
; -/W
.define PSG_LFSR_ENABLE		%10000000
;	7: LFSR Enable flag.
.define PSG_LFSR_FREQMASK	%00011111
;	4-0: LFSR Frequency (~inverted)
; Frequency = 3.58MHz / (64 x ~Value)

.define PSG_LFO_FREQ		$0808
; -/W
;	7-0: LFO Frequency
; Multiplied by the value stored in $0802/$0803 to further
; divide down channel 1's frequency in LFO mode.

.define PSG_LFO_CONT		$0809
; -/W
.define PSG_LFO_CONT_EN		%10000000
;	7: LFO Enable
.define PSG_LFO_CONT_NONE	%00000000
.define PSG_LFO_CONT_ADD_S0	%00000001
.define PSG_LFO_CONT_ADD_S4	%00000010
.define PSG_LFO_CONT_ADD_S8	%00000011
;	1-0: Modulation mode, in all modes other than NONE
; (no modulation) the signed value read from channel 1's
; waveform buffer will be shifted LEFT by Sx and added
; to channel 0's frequency.


;****************************************************************
;			     Timer				*
;****************************************************************
.define TIMER_BACKUP		$0C00
; -R/
;	6-0: Current timer down count.
; -/W
;	6-0: 7-Bit timer reload value. This is decremented at
; (7.16MHz / 1024) = ~6.992KHz regardless of the current CPU
; speed and will trigger an interrupt when it rolls over from
; zero to $7F. The timer will always auto-reload.
;

.define TIMER_EN		$0C01
;	0: Enables the timer interrupt when set to one.


;****************************************************************
;			Gamestick Joyport			*
;****************************************************************
.define GAMESTICK_JOYPORT	$1000
; -R/
;	7: CD-ROM Unit Attached
;	6: Nationality/Type (1 = PC-Engine, 0 = TurboGrafx)
;	5-4: '1'
;	3-0: Joypad Port D3-D0
; -/W
.define	GAMESTICK_CLR		%00000010
.define	GAMESTICK_SEL		%00000001
;	1: Joypad Port CLR
;	0: Joypad Port SEL

; The joypad may be read by lowering CLR and using SEL to
; read the state of the D-Pad and buttons in bits 0-3.
;
; Bit		SEL = 0		SEL = 1
; ----  	--------	--------
; 3		Run		Left
; 2		Select		Right
; 1		Button II/B	Up
; 0		Button I/A	Down
;
; Where '0' indicates a pressed key. These will be rearranged
; into a single byte and inverted for convenience after reading.
.define GAMESTICK_RUN		%10000000
.define GAMESTICK_SELECT	%01000000
.define GAMESTICK_II		%00100000
.define GAMESTICK_I		%00010000

.define GAMESTICK_B		%00100000
.define GAMESTICK_A		%00010000

.define GAMESTICK_LEFT		%00001000
.define GAMESTICK_RIGHT		%00000010
.define GAMESTICK_UP		%00000001
.define GAMESTICK_DOWN		%00000100

; The Turbo Express / PC Engine GT also uses this register for
; bitbanged data transfers over its COM port. This conveniently
; uses a 1/8" stereo cable connected as follows:
;
; Pin		Register (W)
; ----		-------------
; Tip		Bit 0 (SEL)
; Middle	Bit 1 (CLR)
; Inside	GND
;
; Setting CLR high (which also disables the joystick) enables
; reading the state of the COM port pins through $1000:
;
; Pin		Register (R)
; ----		-------------
; Tip		Bit 1 (CLR)
; Middle	Bit 0 (SEL)
;
; For the most basic transfers, CLR can be a clock and SEL data.


;****************************************************************
;			      IRQs				*
;****************************************************************
.define IRQ_EN			$1402
; -R/W
.define IRQTIMER		%00000100
.define IRQ1			%00000010
.define IRQ2			%00000001
;	2: Timer IRQ Disable (when set)
;	1: IRQ1 (From VDC) Disable
;	0: IRQ2 Disable

.define IRQ_STAT		$1403
; -R/W
;	Follows the same bit order as IRQ_EN, reads indicate if
; an IRQ is pending for the possible interrupt sources. Writes
; acknowledge the TIMER interrupt.


;****************************************************************
;			 Useful Macros				*
;****************************************************************
; ADDC [A] [B] [RES]
;	16-Bit add (absolute) with carry.
;----------------------------------------------------------------
.macro ADDC
	LDA.w \1 + LO
	ADC.w \2 + LO
	STA.w \3 + LO
	LDA.w \1 + HI
	ADC.w \2 + HI
	STA.w \3 + HI
.endm

; SBDC [A] [B] [RES]
;	16-Bit subtract (absolute) with carry.
;----------------------------------------------------------------
.macro SBDC
	LDA.w \1 + LO
	SBC.w \2 + LO
	STA.w \3 + LO
	LDA.w \1 + HI
	SBC.w \2 + HI
	STA.w \3 + HI
.endm
